@@ -384,8 +384,16 @@ public:
 	DvbChannel() { }
 	explicit DvbChannel(const DvbChannelBase &channel) : DvbChannelBase(channel) { }
 	~DvbChannel() { }
+
+	DvbChannel &operator=(const DvbChannel &other)
+	{
+		DvbChannelBase *base = this;
+		*base = other;
+		return (*this);
+	}
 };
 
 Q_DECLARE_TYPEINFO(QSharedDataPointer<DvbChannel>, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QExplicitlySharedDataPointer<const DvbChannel>, Q_MOVABLE_TYPE);
 
 #endif /* DVBCHANNEL_H */
