@@ -1,7 +1,7 @@
 /*
  * dvbmanager.h
  *
- * Copyright (C) 2008-2009 Christoph Pfister <christophpfister@gmail.com>
+ * Copyright (C) 2008-2010 Christoph Pfister <christophpfister@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,11 @@
 #define DVBMANAGER_H
 
 #include <QDate>
-#include <QExplicitlySharedDataPointer>
 #include <QMap>
 #include <QPair>
 #include <QStringList>
 
+class QTreeView;
 class DvbBackendDevice;
 class DvbChannelModel;
 class DvbConfig;
@@ -82,6 +82,11 @@ public:
 		return channelModel;
 	}
 
+	QTreeView *getChannelView() const
+	{
+		return channelView;
+	}
+
 	DvbEpgModel *getEpgModel() const
 	{
 		return epgModel;
@@ -97,6 +102,11 @@ public:
 		return recordingModel;
 	}
 
+	void setChannelView(QTreeView *channelView_)
+	{
+		channelView = channelView_;
+	}
+
 	DvbDevice *requestDevice(const QString &source, const DvbTransponder &transponder,
 		RequestType requestType);
 	DvbDevice *requestExclusiveDevice(const QString &source);
@@ -105,7 +115,7 @@ public:
 	QList<DvbDeviceConfig> getDeviceConfigs() const;
 	void updateDeviceConfigs(const QList<DvbDeviceConfigUpdate> &configUpdates);
 
-	QString getScanDataDate(); // returns the formatted short date of the last scan file update
+	QDate getScanDataDate();
 	QStringList getScanSources(TransmissionType type);
 	QString getAutoScanSource(const QString &source) const;
 	QList<DvbTransponder> getTransponders(DvbDevice *device, const QString &source);
@@ -147,6 +157,7 @@ private:
 	QWidget *parent;
 	MediaWidget *mediaWidget;
 	DvbChannelModel *channelModel;
+	QTreeView *channelView;
 	DvbEpgModel *epgModel;
 	DvbLiveView *liveView;
 	DvbRecordingModel *recordingModel;
