@@ -24,21 +24,350 @@
 #include <QBoxLayout>
 #include <QCheckBox>
 #include <QDir>
+#include <QHeaderView>
 #include <QLabel>
 #include <QPushButton>
+#include <QSortFilterProxyModel>
 #include <KAction>
 #include <KCalendarSystem>
 #include <KComboBox>
 #include <KDebug>
 #include <KLineEdit>
+#include <KMessageBox>
 #include <KStandardDirs>
 #include "../datetimeedit.h"
 #include "dvbchannelui.h"
 #include "dvbmanager.h"
 
-DvbRecording::DvbRecording(DvbManager *manager_) : repeat(0), manager(manager_), device(NULL),
-	pmtSection(QByteArray()), pmtValid(false)
+DvbRecordingModel::DvbRecordingModel(DvbManager *manager_, QObject *parent) :
+	QAbstractTableModel(parent), manager(manager_), hasActiveRecordings(false)
 {
+	sqlInterface = new DvbRecordingSqlInterface(manager, this, &recordings, this);
+
+	// we regularly recheck the status of the recordings
+	// this way we can keep retrying if the device was busy / tuning failed
+	// we don't call checkStatus() here, because the devices maybe aren't set up yet
+
+	QTimer *timer = new QTimer(this);
+	connect(timer, SIGNAL(timeout()), this, SLOT(checkStatus()));
+	timer->start(5000);
+
+	// compatibility code
+
+	QFile file(KStandardDirs::locateLocal("appdata", "recordings.dvb"));
+
+	if (!file.exists()) {
+		return;
+	}
+
+	if (!file.open(QIODevice::ReadOnly)) {
+		kWarning() << "cannot open file" << file.fileName();
+		return;
+	}
+
+	QDataStream stream(&file);
+	stream.setVersion(QDataStream::Qt_4_4);
+
+	int version;
+	stream >> version;
+
+	if (version != 0x4d848730) {
+		// the older version didn't store a magic number ...
+		file.seek(0);
+		version = 0;
+	}
+
+	while (!stream.atEnd()) {
+		DvbRecordingEntry entry;
+		stream >> entry.name;
+		stream >> entry.channelName;
+		stream >> entry.begin;
+		entry.begin = entry.begin.toUTC();
+		stream >> entry.duration;
+		QDateTime end;
+		stream >> end;
+
+		if (version != 0) {
+			stream >> entry.repeat;
+		}
+
+		if (stream.status() != QDataStream::Ok) {
+			kWarning() << "invalid recordings in file" << file.fileName();
+			break;
+		}
+
+		scheduleProgram(entry);
+	}
+
+	if (!file.remove()) {
+		kWarning() << "cannot remove file" << file.fileName();
+	}
+}
+
+DvbRecordingModel::~DvbRecordingModel()
+{
+	sqlInterface->flush();
+	qDeleteAll(recordings);
+}
+
+DvbRecordingKey DvbRecordingModel::scheduleProgram(const DvbRecordingEntry &entry)
+{
+	int row = recordings.size();
+	beginInsertRows(QModelIndex(), row, row);
+	recordings.append(new DvbRecording(manager, entry));
+	endInsertRows();
+	QTimer::singleShot(0, this, SLOT(checkStatus()));
+	return DvbRecordingKey(sqlInterface->keyForRow(row));
+}
+
+void DvbRecordingModel::removeProgram(const DvbRecordingKey &key)
+{
+	int row = sqlInterface->rowForKey(key.key);
+
+	if (row >= 0) {
+		removeRow(row);
+	}
+}
+
+QAbstractItemModel *DvbRecordingModel::createProxyModel(QObject *parent)
+{
+	QSortFilterProxyModel *proxyModel = new QSortFilterProxyModel(parent);
+	proxyModel->setDynamicSortFilter(true);
+	proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
+	proxyModel->setSortLocaleAware(true);
+	proxyModel->setSortRole(SortRole);
+	proxyModel->setSourceModel(this);
+	return proxyModel;
+}
+
+void DvbRecordingModel::showDialog(QWidget *parent)
+{
+	KDialog *dialog = new DvbRecordingDialog(manager, this, parent);
+	dialog->setAttribute(Qt::WA_DeleteOnClose, true);
+	dialog->setModal(true);
+	dialog->show();
+}
+
+void DvbRecordingModel::mayCloseApplication(bool *ok, QWidget *parent)
+{
+	if (*ok) {
+		if (hasActiveRecordings) {
+			if (KMessageBox::warningYesNo(parent, i18nc("message box",
+			    "Kaffeine is currently recording programs.\n"
+			    "Do you really want to close the application?")) != KMessageBox::Yes) {
+				*ok = false;
+			}
+
+			return;
+		}
+
+		if (!recordings.isEmpty()) {
+			if (KMessageBox::questionYesNo(parent, i18nc("message box",
+			    "Kaffeine has scheduled recordings.\n"
+			    "Do you really want to close the application?")) != KMessageBox::Yes) {
+				*ok = false;
+			}
+
+			return;
+		}
+	}
+}
+
+void DvbRecordingModel::checkStatus()
+{
+	QDateTime currentLocalDateTime = QDateTime::currentDateTime();
+	QDateTime currentDateTime = currentLocalDateTime.toUTC();
+
+	for (int row = 0; row < recordings.size(); ++row) {
+		DvbRecording *recording = recordings.at(row);
+		const DvbRecordingEntry &entry = recording->getEntry();
+		QDateTime end = entry.begin.addSecs(QTime().secsTo(entry.duration));
+
+		if (end <= currentDateTime) {
+			if (entry.repeat == 0) {
+				removeRow(row);
+				--row;
+			} else {
+				recording->stop();
+
+				// take care of DST switches
+				DvbRecordingEntry nextEntry = entry;
+				nextEntry.begin = nextEntry.begin.toLocalTime();
+				end = end.toLocalTime();
+				int days = end.daysTo(currentLocalDateTime);
+
+				if (end.addDays(days) <= currentLocalDateTime) {
+					++days;
+				}
+
+				// QDate::dayOfWeek() and our dayOfWeek differ by one
+				int dayOfWeek = (nextEntry.begin.date().dayOfWeek() - 1 + days);
+
+				for (int j = 0; j < 7; ++j) {
+					if ((nextEntry.repeat & (1 << (dayOfWeek % 7))) != 0) {
+						break;
+					}
+
+					++days;
+					++dayOfWeek;
+				}
+
+				nextEntry.begin = nextEntry.begin.addDays(days).toUTC();
+				recording->setEntry(nextEntry);
+				emit dataChanged(index(row, 0), index(row, 3));
+			}
+		}
+	}
+
+	hasActiveRecordings = false;
+
+	for (int row = 0; row < recordings.size(); ++row) {
+		DvbRecording *recording = recordings.at(row);
+		const DvbRecordingEntry &entry = recording->getEntry();
+
+		if (entry.begin <= currentDateTime) {
+			hasActiveRecordings = true;
+
+			if (!entry.isRunning) {
+				recording->start();
+
+				if (entry.isRunning) {
+					emit dataChanged(index(row, 0), index(row, 3));
+				}
+			}
+		}
+	}
+}
+
+int DvbRecordingModel::columnCount(const QModelIndex &parent) const
+{
+	if (!parent.isValid()) {
+		return 4;
+	}
+
+	return 0;
+}
+
+int DvbRecordingModel::rowCount(const QModelIndex &parent) const
+{
+	if (!parent.isValid()) {
+		return recordings.size();
+	}
+
+	return 0;
+}
+
+QVariant DvbRecordingModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+	if ((orientation == Qt::Horizontal) && (role == Qt::DisplayRole)) {
+		switch (section) {
+		case 0:
+			return i18n("Name");
+		case 1:
+			return i18n("Channel");
+		case 2:
+			return i18n("Begin");
+		case 3:
+			return i18n("Duration");
+		}
+	}
+
+	return QVariant();
+}
+
+QVariant DvbRecordingModel::data(const QModelIndex &index, int role) const
+{
+	const DvbRecordingEntry &entry = recordings.at(index.row())->getEntry();
+
+	switch (role) {
+	case Qt::DecorationRole:
+		if (index.column() == 0) {
+			if (entry.isRunning) {
+				return KIcon("media-record");
+			} else if (entry.repeat != 0) {
+				return KIcon("view-refresh");
+			}
+		}
+
+		break;
+	case Qt::DisplayRole:
+		switch (index.column()) {
+		case 0:
+			return entry.name;
+		case 1:
+			return entry.channelName;
+		case 2:
+			return KGlobal::locale()->formatDateTime(entry.begin.toLocalTime());
+		case 3:
+			return KGlobal::locale()->formatTime(entry.duration, false, true);
+		}
+
+		break;
+	case SortRole:
+		switch (index.column()) {
+		case 0:
+			return entry.name;
+		case 1:
+			return entry.channelName;
+		case 2:
+			return entry.begin;
+		case 3:
+			return entry.duration;
+		}
+
+		break;
+	case DvbRecordingEntryRole:
+		return QVariant::fromValue(&entry);
+	}
+
+	return QVariant();
+}
+
+bool DvbRecordingModel::removeRows(int row, int count, const QModelIndex &parent)
+{
+	Q_UNUSED(parent)
+	QList<quint32> keys;
+
+	for (int currentRow = row; currentRow < (row + count); ++currentRow) {
+		keys.append(sqlInterface->keyForRow(currentRow));
+	}
+
+	beginRemoveRows(QModelIndex(), row, row + count - 1);
+	qDeleteAll(recordings.begin() + row, recordings.begin() + row + count);
+	recordings.erase(recordings.begin() + row, recordings.begin() + row + count);
+	endRemoveRows();
+
+	foreach (quint32 key, keys) {
+		emit programRemoved(DvbRecordingKey(key));
+	}
+
+	return true;
+}
+
+bool DvbRecordingModel::setData(const QModelIndex &modelIndex, const QVariant &value, int role)
+{
+	Q_UNUSED(role)
+	const DvbRecordingEntry *entry = value.value<const DvbRecordingEntry *>();
+
+	if (entry != NULL) {
+		if (modelIndex.isValid()) {
+			int row = modelIndex.row();
+			recordings.at(row)->setEntry(*entry);
+			emit dataChanged(index(row, 0), index(row, 3));
+		} else {
+			scheduleProgram(*entry);
+		}
+
+		return true;
+	}
+
+	return false;
+}
+
+DvbRecording::DvbRecording(DvbManager *manager_, const DvbRecordingEntry &entry_) :
+	manager(manager_), device(NULL), pmtSection(QByteArray()), pmtValid(false)
+{
+	setEntry(entry_);
 	connect(&pmtFilter, SIGNAL(pmtSectionChanged(DvbPmtSection)),
 		this, SLOT(pmtSectionChanged(DvbPmtSection)));
 	connect(&patPmtTimer, SIGNAL(timeout()), this, SLOT(insertPatPmt()));
@@ -49,27 +378,47 @@ DvbRecording::~DvbRecording()
 	stop();
 }
 
-bool DvbRecording::isRunning() const
+const DvbRecordingEntry &DvbRecording::getEntry() const
 {
-	return (device != NULL);
+	return entry;
+}
+
+void DvbRecording::setEntry(const DvbRecordingEntry &entry_)
+{
+	entry.name = entry_.name;
+	entry.channelName = entry_.channelName;
+	entry.begin = entry_.begin;
+	entry.duration = entry_.duration;
+	entry.repeat = entry_.repeat;
+	// isRunning is read-only
+
+	if (entry.begin.timeSpec() != Qt::UTC) {
+		kWarning() << "wrong time spec";
+		entry.begin = entry.begin.toUTC();
+	}
+
+	// the seconds and milliseconds aren't visible --> set them to zero
+	entry.begin = entry.begin.addMSecs(-(QTime().msecsTo(entry.begin.time()) % 60000));
 }
 
 void DvbRecording::start()
 {
 	if (channel.constData() == NULL) {
-		int row = manager->getChannelModel()->indexOfName(channelName);
+		DvbChannelModel *channelModel = manager->getChannelModel();
+		QModelIndex index = channelModel->findChannelByName(entry.channelName);
 
-		if (row < 0) {
-			kWarning() << "cannot find channel" << channelName;
+		if (!index.isValid()) {
+			kWarning() << "cannot find channel" << entry.channelName;
 			return;
 		}
 
-		channel = manager->getChannelModel()->getChannel(row);
+		channel = channelModel->data(index,
+			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
 	}
 
 	if (!file.isOpen()) {
 		QString folder = manager->getRecordingFolder();
-		QString path = folder + '/' + name.replace('/', '_');
+		QString path = folder + '/' + entry.name.replace('/', '_');
 
 		for (int attempt = 0; attempt < 100; ++attempt) {
 			if (attempt == 0) {
@@ -95,7 +444,7 @@ void DvbRecording::start()
 
 			if (folder != QDir::homePath()) {
 				folder = QDir::homePath();
-				path = folder + '/' + name.replace('/', '_');
+				path = folder + '/' + entry.name.replace('/', '_');
 				attempt = -1;
 				continue;
 			}
@@ -129,10 +478,14 @@ void DvbRecording::start()
 			device->startDescrambling(pmtSection, this);
 		}
 	}
+
+	entry.isRunning = true;
 }
 
 void DvbRecording::stop()
 {
+	entry.isRunning = false;
+
 	if (device != NULL) {
 		if (channel->isScrambled && pmtSection.isValid()) {
 			device->stopDescrambling(pmtSection.programNumber(), this);
@@ -289,341 +642,138 @@ void DvbRecording::processData(const char data[188])
 	file.write(data, 188);
 }
 
-DvbRecordingModel::DvbRecordingModel(DvbManager *manager_) : QAbstractTableModel(manager_),
-	manager(manager_), scheduleDialog(NULL), treeView(NULL)
+DvbRecordingSqlInterface::DvbRecordingSqlInterface(DvbManager *manager_, QAbstractItemModel *model,
+	QList<DvbRecording *> *recordings_, QObject *parent) : SqlTableModelInterface(parent),
+	manager(manager_), recordings(recordings_)
 {
-	sqlInterface = new DvbSqlRecordingModelInterface(manager, this);
-
-	// we regularly recheck the status of the recordings
-	// this way we can keep retrying if the device was busy / tuning failed
-
-	checkStatusTimer.start(5000);
-	connect(&checkStatusTimer, SIGNAL(timeout()), this, SLOT(checkStatus()));
-
-	// we don't call checkStatus() here, because the devices maybe aren't set up yet
-
-	// compatibility code
-
-	QFile file(KStandardDirs::locateLocal("appdata", "recordings.dvb"));
-
-	if (!file.exists()) {
-		return;
-	}
-
-	if (!file.open(QIODevice::ReadOnly)) {
-		kWarning() << "cannot open file" << file.fileName();
-		return;
-	}
-
-	QDataStream stream(&file);
-	stream.setVersion(QDataStream::Qt_4_4);
-
-	int version;
-	stream >> version;
-
-	if (version != 0x4d848730) {
-		// the older version didn't store a magic number ...
-		file.seek(0);
-		version = 0;
-	}
-
-	while (!stream.atEnd()) {
-		DvbRecording *recording = new DvbRecording(manager);
-		stream >> recording->name;
-		stream >> recording->channelName;
-		stream >> recording->begin;
-		recording->begin = recording->begin.toLocalTime();
-		stream >> recording->duration;
-		QDateTime end;
-		stream >> end;
-		recording->end = recording->begin.addSecs(QTime().secsTo(recording->duration));
-
-		if (version != 0) {
-			stream >> recording->repeat;
-		}
-
-		if (stream.status() != QDataStream::Ok) {
-			kWarning() << "invalid recordings in file" << file.fileName();
-			delete recording;
-			break;
-		}
-
-		insert(recording);
-	}
-
-	if (!file.remove()) {
-		kWarning() << "cannot remove file" << file.fileName();
-	}
+	init(model, "RecordingSchedule",
+		QStringList() << "Name" << "Channel" << "Begin" << "Duration" << "Repeat");
 }
 
-DvbRecordingModel::~DvbRecordingModel()
+DvbRecordingSqlInterface::~DvbRecordingSqlInterface()
 {
-	sqlInterface->flush();
-	qDeleteAll(recordings);
 }
 
-DvbRecordingIndex DvbRecordingModel::scheduleProgram(const QString &name, const QString &channel,
-	const QDateTime &begin, const QTime &duration)
+int DvbRecordingSqlInterface::insertFromSqlQuery(const QSqlQuery &query, int index)
 {
-	Q_ASSERT(begin.timeSpec() == Qt::LocalTime);
-	DvbRecording *recording = new DvbRecording(manager);
-	recording->name = name;
-	recording->channelName = channel;
-	recording->begin = begin;
-	// the seconds aren't visible --> set them to zero
-	recording->begin = recording->begin.addSecs(-recording->begin.time().second());
-	recording->duration = duration;
-	recording->end = recording->begin.addSecs(QTime().secsTo(recording->duration));
-	return insert(recording);
-}
+	DvbRecordingEntry entry;
+	entry.name = query.value(index++).toString();
+	entry.channelName = query.value(index++).toString();
+	entry.begin = QDateTime::fromString(query.value(index++).toString(), Qt::ISODate).toUTC();
+	entry.duration = QTime::fromString(query.value(index++).toString(), Qt::ISODate);
+	entry.repeat = (query.value(index++).toInt() & ((1 << 7) - 1));
 
-void DvbRecordingModel::removeProgram(const DvbRecordingIndex &recordingIndex)
-{
-	int row = sqlInterface->rowForKey(recordingIndex.key);
-
-	if (row >= 0) {
-		remove(row);
+	if (!entry.name.isEmpty() && !entry.channelName.isEmpty() &&
+	    entry.begin.isValid() && entry.duration.isValid()) {
+		int row = recordings->size();
+		recordings->append(new DvbRecording(manager, entry));
+		return row;
 	}
+
+	return -1;
 }
 
-void DvbRecordingModel::showDialog(QWidget *parent)
+void DvbRecordingSqlInterface::bindToSqlQuery(QSqlQuery &query, int index, int row) const
 {
-	scheduleDialog = new KDialog(parent);
-	scheduleDialog->setButtons(KDialog::Close);
-	scheduleDialog->setCaption(i18nc("dialog", "Recording Schedule"));
+	const DvbRecordingEntry &entry = recordings->at(row)->getEntry();
+	query.bindValue(index++, entry.name);
+	query.bindValue(index++, entry.channelName);
+	query.bindValue(index++, entry.begin.toString(Qt::ISODate) + 'Z');
+	query.bindValue(index++, entry.duration.toString(Qt::ISODate));
+	query.bindValue(index++, entry.repeat);
+}
 
-	QWidget *widget = new QWidget(scheduleDialog);
+DvbRecordingDialog::DvbRecordingDialog(DvbManager *manager_, DvbRecordingModel *recordingModel,
+	QWidget *parent) : KDialog(parent), manager(manager_)
+{
+	setButtons(KDialog::Close);
+	setCaption(i18nc("dialog", "Recording Schedule"));
+
+	QWidget *widget = new QWidget(this);
 	QBoxLayout *mainLayout = new QVBoxLayout(widget);
 	QBoxLayout *boxLayout = new QHBoxLayout();
 
-	QPushButton *pushButton = new QPushButton(KIcon("list-add"),
-		i18nc("add a new item to a list", "New"), widget);
+	proxyModel = recordingModel->createProxyModel(widget);
+	treeView = new QTreeView(widget);
+	treeView->header()->setResizeMode(QHeaderView::ResizeToContents);
+	treeView->setContextMenuPolicy(Qt::ActionsContextMenu);
+	treeView->setModel(proxyModel);
+	treeView->setRootIsDecorated(false);
+	treeView->setSelectionMode(QAbstractItemView::ExtendedSelection);
+	treeView->sortByColumn(2, Qt::AscendingOrder);
+	treeView->setSortingEnabled(true);
+
+	KAction *action =
+		new KAction(KIcon("list-add"), i18nc("add a new item to a list", "New"), widget);
+	connect(action, SIGNAL(triggered()), this, SLOT(newRecording()));
+	treeView->addAction(action);
+	QPushButton *pushButton = new QPushButton(action->icon(), action->text(), widget);
 	connect(pushButton, SIGNAL(clicked()), this, SLOT(newRecording()));
 	boxLayout->addWidget(pushButton);
 
-	pushButton = new QPushButton(KIcon("configure"), i18n("Edit"), widget);
+	action = new KAction(KIcon("configure"), i18n("Edit"), widget);
+	connect(action, SIGNAL(triggered()), this, SLOT(editRecording()));
+	treeView->addAction(action);
+	pushButton = new QPushButton(action->icon(), action->text(), widget);
 	connect(pushButton, SIGNAL(clicked()), this, SLOT(editRecording()));
 	boxLayout->addWidget(pushButton);
 
-	pushButton = new QPushButton(KIcon("edit-delete"),
+	action = new KAction(KIcon("edit-delete"),
 		i18nc("remove an item from a list", "Remove"), widget);
+	connect(action, SIGNAL(triggered()), this, SLOT(removeRecording()));
+	treeView->addAction(action);
+	pushButton = new QPushButton(action->icon(), action->text(), widget);
 	connect(pushButton, SIGNAL(clicked()), this, SLOT(removeRecording()));
 	boxLayout->addWidget(pushButton);
 
-	boxLayout->addStretch(1);
+	boxLayout->addStretch();
 	mainLayout->addLayout(boxLayout);
-
-	treeView = new ProxyTreeView(widget);
-	treeView->setModel(this);
-	treeView->setRootIsDecorated(false);
-	treeView->sortByColumn(2, Qt::AscendingOrder);
-	treeView->setSortingEnabled(true);
 	mainLayout->addWidget(treeView);
-
-	KAction *action = new KAction(KIcon("configure"), i18n("Edit"), widget);
-	connect(action, SIGNAL(triggered()), this, SLOT(editRecording()));
-	treeView->addAction(action);
-
-	action = new KAction(KIcon("edit-delete"), i18nc("remove an item from a list", "Remove"),
-		widget);
-	connect(action, SIGNAL(triggered()), this, SLOT(removeRecording()));
-	treeView->addAction(action);
-
-	scheduleDialog->setMainWidget(widget);
-	scheduleDialog->setAttribute(Qt::WA_DeleteOnClose, true);
-	scheduleDialog->setModal(true);
-	scheduleDialog->show();
+	setMainWidget(widget);
+	resize(100 * fontMetrics().averageCharWidth(), 20 * fontMetrics().height());
 }
 
-int DvbRecordingModel::columnCount(const QModelIndex &parent) const
+DvbRecordingDialog::~DvbRecordingDialog()
 {
-	if (!parent.isValid()) {
-		return 4;
-	}
-
-	return 0;
 }
 
-int DvbRecordingModel::rowCount(const QModelIndex &parent) const
+void DvbRecordingDialog::newRecording()
 {
-	if (!parent.isValid()) {
-		return recordings.size();
-	}
-
-	return 0;
-}
-
-QVariant DvbRecordingModel::headerData(int section, Qt::Orientation orientation, int role) const
-{
-	if ((role == Qt::DisplayRole) && (orientation == Qt::Horizontal)) {
-		switch (section) {
-		case 0:
-			return i18n("Name");
-		case 1:
-			return i18n("Channel");
-		case 2:
-			return i18n("Begin");
-		case 3:
-			return i18n("Duration");
-		}
-	}
-
-	return QVariant();
-}
-
-QVariant DvbRecordingModel::data(const QModelIndex &index, int role) const
-{
-	switch (role) {
-	case Qt::DecorationRole:
-		if (index.column() == 0) {
-			const DvbRecording *recording = recordings.at(index.row());
-
-			if (recording->isRunning()) {
-				return KIcon("media-record");
-			} else if (recording->repeat != 0) {
-				return KIcon("view-refresh");
-			}
-		}
-
-		break;
-	case Qt::DisplayRole: {
-		const DvbRecording *recording = recordings.at(index.row());
-
-		switch (index.column()) {
-		case 0:
-			return recording->name;
-		case 1:
-			return recording->channelName;
-		case 2:
-			return KGlobal::locale()->formatDateTime(recording->begin);
-		case 3:
-			return KGlobal::locale()->formatTime(recording->duration, false, true);
-		}
-
-		break;
-	    }
-	}
-
-	return QVariant();
-}
-
-DvbRecordingIndex DvbRecordingModel::insert(DvbRecording *recording)
-{
-	int row = recordings.size();
-	beginInsertRows(QModelIndex(), row, row);
-	recordings.insert(row, recording);
-	endInsertRows();
-	checkStatus();
-	return DvbRecordingIndex(sqlInterface->keyForRow(row));
-}
-
-void DvbRecordingModel::update(int row)
-{
-	emit dataChanged(index(row, 0), index(row, 3));
-	checkStatus();
-}
-
-void DvbRecordingModel::remove(int row)
-{
-	DvbRecordingIndex recordingIndex(sqlInterface->keyForRow(row));
-	beginRemoveRows(QModelIndex(), row, row);
-	delete recordings.at(row);
-	recordings.removeAt(row);
-	endRemoveRows();
-	emit programRemoved(recordingIndex);
-}
-
-void DvbRecordingModel::checkStatus()
-{
-	QDateTime current = QDateTime::currentDateTime();
-
-	for (int row = 0; row < recordings.size(); ++row) {
-		DvbRecording *recording = recordings.at(row);
-
-		if (recording->end <= current) {
-			if (recording->repeat == 0) {
-				remove(row);
-				--row;
-			} else {
-				recording->stop();
-				int days = recording->end.daysTo(current);
-
-				if (recording->end.addDays(days) <= current) {
-					++days;
-				}
-
-				// QDate::dayOfWeek() and our dayOfWeek differ by one
-				int dayOfWeek = recording->begin.date().dayOfWeek() - 1 + days;
-
-				for (int j = 0; j < 7; ++j) {
-					if ((recording->repeat & (1 << (dayOfWeek % 7))) != 0) {
-						break;
-					}
-
-					++days;
-					++dayOfWeek;
-				}
-
-				recording->begin = recording->begin.addDays(days);
-				recording->end = recording->end.addDays(days);
-				emit dataChanged(index(row, 0), index(row, 3));
-			}
-		}
-	}
-
-	for (int row = 0; row < recordings.size(); ++row) {
-		DvbRecording *recording = recordings.at(row);
-
-		if (recording->begin <= current) {
-			if (recording->isRunning()) {
-				continue;
-			}
-
-			recording->start();
-
-			if (recording->isRunning()) {
-				emit dataChanged(index(row, 0), index(row, 3));
-			}
-		}
-	}
-}
-
-void DvbRecordingModel::newRecording()
-{
-	KDialog *dialog = new DvbRecordingEditor(manager, this, -1, scheduleDialog);
+	KDialog *dialog = new DvbRecordingEditor(manager, proxyModel, QModelIndex(), this);
 	dialog->setAttribute(Qt::WA_DeleteOnClose, true);
 	dialog->setModal(true);
 	dialog->show();
 }
 
-void DvbRecordingModel::editRecording()
+void DvbRecordingDialog::editRecording()
 {
-	int row = treeView->selectedRow();
+	QModelIndex index = treeView->currentIndex();
 
-	if (row >= 0) {
-		KDialog *dialog = new DvbRecordingEditor(manager, this, row, scheduleDialog);
+	if (index.isValid()) {
+		KDialog *dialog = new DvbRecordingEditor(manager, proxyModel, index, this);
 		dialog->setAttribute(Qt::WA_DeleteOnClose, true);
 		dialog->setModal(true);
 		dialog->show();
 	}
 }
 
-void DvbRecordingModel::removeRecording()
+void DvbRecordingDialog::removeRecording()
 {
-	int row = treeView->selectedRow();
+	QMap<int, char> selectedRows;
 
-	if (row >= 0) {
-		remove(row);
+	foreach (const QModelIndex &modelIndex, treeView->selectionModel()->selectedIndexes()) {
+		selectedRows.insert(modelIndex.row(), 0);
+	}
+
+	for (QMap<int, char>::ConstIterator it = selectedRows.end(); it != selectedRows.begin();) {
+		--it;
+		proxyModel->removeRow(it.key());
 	}
 }
 
-static bool localeAwareLessThan3(const QString &x, const QString &y)
-{
-	return (x.localeAwareCompare(y) < 0);
-}
-
-DvbRecordingEditor::DvbRecordingEditor(DvbManager *manager_, DvbRecordingModel *model_, int row_,
-	QWidget *parent) : KDialog(parent), manager(manager_), model(model_), row(row_)
+DvbRecordingEditor::DvbRecordingEditor(DvbManager *manager, QAbstractItemModel *model_,
+	const QModelIndex &modelIndex_, QWidget *parent) : KDialog(parent), model(model_),
+	persistentIndex(modelIndex_)
 {
 	setCaption(i18nc("subdialog of 'Recording Schedule'", "Edit Schedule Entry"));
 
@@ -631,25 +781,20 @@ DvbRecordingEditor::DvbRecordingEditor(DvbManager *manager_, DvbRecordingModel *
 	QGridLayout *gridLayout = new QGridLayout(widget);
 
 	nameEdit = new KLineEdit(widget);
-	connect(nameEdit, SIGNAL(textChanged(QString)), this, SLOT(checkValid()));
+	connect(nameEdit, SIGNAL(textChanged(QString)), this, SLOT(checkValidity()));
 	gridLayout->addWidget(nameEdit, 0, 1);
 
 	QLabel *label = new QLabel(i18n("Name:"), widget);
 	label->setBuddy(nameEdit);
 	gridLayout->addWidget(label, 0, 0);
 
-	QStringList channels; // FIXME use proxy model?
-
-	foreach (const QSharedDataPointer<DvbChannel> &channel,
-		 manager->getChannelModel()->getChannels()) {
-		channels.append(channel->name);
-	}
-
-	qSort(channels.begin(), channels.end(), localeAwareLessThan3);
-
 	channelBox = new KComboBox(widget);
-	channelBox->addItems(channels);
-	connect(channelBox, SIGNAL(currentIndexChanged(int)), this, SLOT(checkValid()));
+	QAbstractProxyModel *channelProxyModel =
+		manager->getChannelModel()->createProxyModel(channelBox);
+	QHeaderView *header = manager->getChannelView()->header();
+	channelProxyModel->sort(header->sortIndicatorSection(), header->sortIndicatorOrder());
+	channelBox->setModel(channelProxyModel);
+	connect(channelBox, SIGNAL(currentIndexChanged(int)), this, SLOT(checkValidity()));
 	gridLayout->addWidget(channelBox, 1, 1);
 
 	label = new QLabel(i18n("Channel:"), widget);
@@ -686,8 +831,8 @@ DvbRecordingEditor::DvbRecordingEditor(DvbManager *manager_, DvbRecordingModel *
 	gridLayout->addWidget(new QLabel(i18n("Repeat:"), widget), 5, 0);
 
 	QBoxLayout *boxLayout = new QHBoxLayout();
-	QPushButton *pushButton = new QPushButton(
-		i18nc("button next to the 'Repeat:' label", "Never"), widget);
+	QPushButton *pushButton =
+		new QPushButton(i18nc("button next to the 'Repeat:' label", "Never"), widget);
 	connect(pushButton, SIGNAL(clicked()), this, SLOT(repeatNever()));
 	boxLayout->addWidget(pushButton);
 
@@ -708,33 +853,33 @@ DvbRecordingEditor::DvbRecordingEditor(DvbManager *manager_, DvbRecordingModel *
 	gridLayout->addLayout(dayLayout, 6, 1);
 	setMainWidget(widget);
 
-	if (row >= 0) {
-		const DvbRecording *recording = model->recordings.at(row);
-		nameEdit->setText(recording->name);
-		channelBox->setCurrentIndex(channels.indexOf(recording->channelName));
-		beginEdit->setDateTime(recording->begin);
-		durationEdit->setTime(recording->duration);
+	if (persistentIndex.isValid()) {
+		const DvbRecordingEntry *entry =
+			model->data(persistentIndex, DvbRecordingModel::DvbRecordingEntryRole).
+			value<const DvbRecordingEntry *>();
+		nameEdit->setText(entry->name);
+		channelBox->setCurrentIndex(channelBox->findText(entry->channelName));
+		beginEdit->setDateTime(entry->begin.toLocalTime());
+		durationEdit->setTime(entry->duration);
 
 		for (int i = 0; i < 7; ++i) {
-			if ((recording->repeat & (1 << i)) != 0) {
+			if ((entry->repeat & (1 << i)) != 0) {
 				dayCheckBoxes[i]->setChecked(true);
 			}
 		}
 
-		if (recording->isRunning()) {
+		if (entry->isRunning) {
 			nameEdit->setEnabled(false);
 			channelBox->setEnabled(false);
 			beginEdit->setEnabled(false);
 		}
 	} else {
 		channelBox->setCurrentIndex(-1);
-		QDateTime begin = QDateTime::currentDateTime();
-		// the seconds aren't visible --> set them to zero
-		beginEdit->setDateTime(begin.addSecs(-begin.time().second()));
+		beginEdit->setDateTime(QDateTime::currentDateTime());
 		durationEdit->setTime(QTime(2, 0));
 	}
 
-	checkValid();
+	checkValidity();
 
 	if (nameEdit->text().isEmpty()) {
 		nameEdit->setFocus();
@@ -777,82 +922,28 @@ void DvbRecordingEditor::repeatDaily()
 	}
 }
 
-void DvbRecordingEditor::checkValid()
+void DvbRecordingEditor::checkValidity()
 {
 	enableButtonOk(!nameEdit->text().isEmpty() && (channelBox->currentIndex() != -1));
 }
 
 void DvbRecordingEditor::accept()
 {
-	DvbRecording *recording;
-
-	if (row >= 0) {
-		recording = model->recordings.at(row);
-	} else {
-		recording = new DvbRecording(manager);
-	}
-
-	recording->name = nameEdit->text();
-	recording->channelName = channelBox->currentText();
-	recording->begin = beginEdit->dateTime();
-	recording->duration = durationEdit->time();
-	recording->end = endEdit->dateTime();
-	recording->repeat = 0;
+	DvbRecordingEntry entry;
+	entry.name = nameEdit->text();
+	entry.channelName = channelBox->currentText();
+	entry.begin = beginEdit->dateTime().toUTC();
+	entry.duration = durationEdit->time();
 
 	for (int i = 0; i < 7; ++i) {
 		if (dayCheckBoxes[i]->isChecked()) {
-			recording->repeat |= (1 << i);
+			entry.repeat |= (1 << i);
 		}
 	}
 
-	if (row >= 0) {
-		model->update(row);
-	} else {
-		model->insert(recording);
-	}
+	// if persistentIndex is invalid, the entry is appended
+	const DvbRecordingEntry *constEntry = &entry;
+	model->setData(persistentIndex, QVariant::fromValue(constEntry));
 
 	KDialog::accept();
 }
-
-DvbSqlRecordingModelInterface::DvbSqlRecordingModelInterface(DvbManager *manager_,
-	DvbRecordingModel *model_) : SqlTableModelInterface(model_), manager(manager_),
-	model(model_)
-{
-	init(model, "RecordingSchedule",
-		QStringList() << "Name" << "Channel" << "Begin" << "Duration" << "Repeat");
-}
-
-DvbSqlRecordingModelInterface::~DvbSqlRecordingModelInterface()
-{
-}
-
-int DvbSqlRecordingModelInterface::insertFromSqlQuery(const QSqlQuery &query, int index)
-{
-	DvbRecording *recording = new DvbRecording(manager);
-	recording->name = query.value(index++).toString();
-	recording->channelName = query.value(index++).toString();
-	recording->begin = QDateTime::fromString(query.value(index++).toString(), Qt::ISODate);
-	recording->duration = QTime::fromString(query.value(index++).toString(), Qt::ISODate);
-	recording->end = recording->begin.addSecs(QTime().secsTo(recording->duration));
-	recording->repeat = (query.value(index++).toInt() & ((1 << 7) - 1));
-
-	if (recording->name.isEmpty() || recording->channelName.isEmpty() ||
-	    !recording->begin.isValid() || !recording->duration.isValid()) {
-		delete recording;
-		return -1;
-	}
-
-	int row = model->recordings.size();
-	model->recordings.insert(row, recording);
-	return row;
-}
-
-void DvbSqlRecordingModelInterface::bindToSqlQuery(QSqlQuery &query, int index, int row) const
-{
-	const DvbRecording *recording = model->recordings.at(row);
-	query.bindValue(index++, recording->name);
-	query.bindValue(index++, recording->channelName);
-	query.bindValue(index++, recording->begin.toString(Qt::ISODate));
-	query.bindValue(index++, recording->duration.toString(Qt::ISODate));
-	query.bindValue(index++, recording->repeat);
-}
