@@ -20,6 +20,7 @@
 
 #include "dvbtab.h"
 
+#include <QAbstractProxyModel>
 #include <QBoxLayout>
 #include <QDir>
 #include <QHeaderView>
@@ -28,6 +29,7 @@
 #include <QToolButton>
 #include <KAction>
 #include <KActionCollection>
+#include <KDebug>
 #include <KLineEdit>
 #include <KLocale>
 #include <KMenu>
@@ -117,8 +119,8 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 	connect(mediaWidget, SIGNAL(previousDvbChannel()), this, SLOT(previousChannel()));
 	connect(mediaWidget, SIGNAL(nextDvbChannel()), this, SLOT(nextChannel()));
 
-	connect(manager->getRecordingModel(), SIGNAL(programRemoved(DvbRecordingIndex)),
-		this, SLOT(programRemoved(DvbRecordingIndex)));
+	connect(manager->getRecordingModel(), SIGNAL(programRemoved(DvbRecordingKey)),
+		this, SLOT(programRemoved(DvbRecordingKey)));
 
 	QBoxLayout *boxLayout = new QHBoxLayout(this);
 	boxLayout->setMargin(0);
@@ -137,9 +139,10 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 	boxLayout->addWidget(lineEdit);
 	leftLayout->addLayout(boxLayout);
 
-	DvbChannelModel *channelModel = manager->getChannelModel();
-	channelView = new DvbChannelView(channelModel, leftWidget);
-	channelView->setModel(channelModel);
+	channelView = new DvbChannelView(leftWidget);
+	channelView->setContextMenuPolicy(Qt::ActionsContextMenu);
+	channelProxyModel = manager->getChannelModel()->createProxyModel(channelView);
+	channelView->setModel(channelProxyModel);
 	channelView->setRootIsDecorated(false);
 
 	if (!channelView->header()->restoreState(QByteArray::fromBase64(
@@ -148,9 +151,11 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 	}
 
 	channelView->setSortingEnabled(true);
+	channelView->addEditAction();
 	connect(channelView, SIGNAL(activated(QModelIndex)), this, SLOT(playChannel(QModelIndex)));
 	connect(lineEdit, SIGNAL(textChanged(QString)),
-		channelView->model(), SLOT(setFilterRegExp(QString)));
+		channelProxyModel, SLOT(setFilterRegExp(QString)));
+	manager->setChannelView(channelView);
 	leftLayout->addWidget(channelView);
 
 	boxLayout = new QHBoxLayout();
@@ -217,12 +222,22 @@ DvbTab::~DvbTab()
 
 void DvbTab::playChannel(const QString &nameOrNumber)
 {
+	DvbChannelModel *channelModel = manager->getChannelModel();
+	QModelIndex index;
 	int number = nameOrNumber.toInt();
 
-	if (number <= 0) {
-		playChannel(manager->getChannelModel()->indexOfName(nameOrNumber));
-	} else {
-		playChannel(manager->getChannelModel()->indexOfNumber(number));
+	if (number > 0) {
+		index = channelModel->findChannelByNumber(number);
+	}
+
+	if (!index.isValid()) {
+		index = channelModel->findChannelByName(nameOrNumber);
+	}
+
+	if (index.isValid()) {
+		const DvbChannel *channel = channelModel->data(index,
+			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
+		playChannel(channel, channelProxyModel->mapFromSource(index));
 	}
 }
 
@@ -232,7 +247,14 @@ void DvbTab::playLastChannel()
 		lastChannel = currentChannel;
 	}
 
-	playChannel(manager->getChannelModel()->indexOfName(lastChannel));
+	DvbChannelModel *channelModel = manager->getChannelModel();
+	QModelIndex index = channelModel->findChannelByName(lastChannel);
+
+	if (index.isValid()) {
+		const DvbChannel *channel = channelModel->data(index,
+			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
+		playChannel(channel, channelProxyModel->mapFromSource(index));
+	}
 }
 
 void DvbTab::toggleOsd()
@@ -259,6 +281,11 @@ void DvbTab::osdKeyPressed(int key)
 	}
 }
 
+void DvbTab::mayCloseApplication(bool *ok, QWidget *parent)
+{
+	manager->getRecordingModel()->mayCloseApplication(ok, parent);
+}
+
 void DvbTab::showChannelDialog()
 {
 	KDialog *dialog = new DvbScanDialog(manager, this);
@@ -297,22 +324,25 @@ void DvbTab::instantRecord(bool checked)
 			return;
 		}
 
+		DvbRecordingEntry recordingEntry;
 		// FIXME use epg for name
-		instantRecordingIndex = manager->getRecordingModel()->scheduleProgram(
-			channel->name + QTime::currentTime().toString("-hhmmss"), channel->name,
-			QDateTime::currentDateTime(), QTime(12, 0));
-
+		recordingEntry.name = (channel->name + QTime::currentTime().toString("-hhmmss"));
+		recordingEntry.channelName = channel->name;
+		recordingEntry.begin = QDateTime::currentDateTime().toUTC();
+		recordingEntry.duration = QTime(12, 0);
+		instantRecordingKey =
+			manager->getRecordingModel()->scheduleProgram(recordingEntry);
 		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Started"), 1500);
 	} else {
-		manager->getRecordingModel()->removeProgram(instantRecordingIndex);
+		manager->getRecordingModel()->removeProgram(instantRecordingKey);
 		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Stopped"), 1500);
 	}
 }
 
-void DvbTab::programRemoved(const DvbRecordingIndex &recordingIndex)
+void DvbTab::programRemoved(const DvbRecordingKey &recordingKey)
 {
-	if (instantRecordingIndex == recordingIndex) {
-		instantRecordingIndex = DvbRecordingIndex();
+	if (instantRecordingKey == recordingKey) {
+		instantRecordingKey = DvbRecordingKey();
 		instantRecordAction->setChecked(false);
 		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Stopped"), 1500);
 	}
@@ -328,36 +358,44 @@ void DvbTab::configureDvb()
 
 void DvbTab::tuneOsdChannel()
 {
-	int row = manager->getChannelModel()->indexOfNumber(osdChannel.toInt());
+	int number = osdChannel.toInt();
 	osdChannel.clear();
 	osdChannelTimer.stop();
-	playChannel(row);
+
+	DvbChannelModel *channelModel = manager->getChannelModel();
+	QModelIndex index = channelModel->findChannelByNumber(number);
+
+	if (index.isValid()) {
+		const DvbChannel *channel = channelModel->data(index,
+			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
+		playChannel(channel, channelProxyModel->mapFromSource(index));
+	}
 }
 
 void DvbTab::playChannel(const QModelIndex &index)
 {
-	playChannel(channelView->mapToSource(index));
+	if (index.isValid()) {
+		const DvbChannel *channel = channelProxyModel->data(index,
+			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
+		playChannel(channel, index);
+	}
 }
 
 void DvbTab::previousChannel()
 {
-	// it's enough to look at a single element if you can only select a single row
 	QModelIndex index = channelView->currentIndex();
 
 	if (index.isValid()) {
-		playChannel(channelView->mapToSource(
-			index.sibling(index.row() - 1, index.column())));
+		playChannel(index.sibling(index.row() - 1, index.column()));
 	}
 }
 
 void DvbTab::nextChannel()
 {
-	// it's enough to look at a single element if you can only select a single row
 	QModelIndex index = channelView->currentIndex();
 
 	if (index.isValid()) {
-		playChannel(channelView->mapToSource(
-			index.sibling(index.row() + 1, index.column())));
+		playChannel(index.sibling(index.row() + 1, index.column()));
 	}
 }
 
@@ -385,9 +423,10 @@ void DvbTab::activate()
 	mediaWidget->setFocus();
 }
 
-void DvbTab::playChannel(int row)
+void DvbTab::playChannel(const DvbChannel *channel, const QModelIndex &index)
 {
-	if (row < 0) {
+	if (channel == NULL) {
+		kWarning() << "channel is invalid";
 		return;
 	}
 
@@ -395,13 +434,7 @@ void DvbTab::playChannel(int row)
 		lastChannel = currentChannel;
 	}
 
-	QModelIndex index = channelView->mapFromSource(manager->getChannelModel()->index(row, 0));
-
-	if (index.isValid()) {
-		channelView->setCurrentIndex(index);
-	}
-
-	const DvbChannel *channel = manager->getChannelModel()->getChannel(row);
+	channelView->setCurrentIndex(index);
 	currentChannel = channel->name;
 	manager->getLiveView()->playChannel(channel);
 }
