@@ -83,12 +83,12 @@ StartTab::StartTab(MainWindow *mainWindow)
 	connect(button, SIGNAL(clicked()), mainWindow, SLOT(openDvd()));
 	gridLayout->addWidget(button, 1, 0);
 
-#ifdef HAVE_DVB
+#if HAVE_DVB == 1
 	button = addShortcut(i18n("&5 Digital TV"), KIcon("video-television"), this);
 	button->setShortcut(Qt::Key_5);
 	connect(button, SIGNAL(clicked()), mainWindow, SLOT(playDvb()));
 	gridLayout->addWidget(button, 1, 1);
-#endif /* HAVE_DVB */
+#endif /* HAVE_DVB == 1 */
 }
 
 QAbstractButton *StartTab::addShortcut(const QString &name, const KIcon &icon, QWidget *parent)
@@ -225,10 +225,10 @@ MainWindow::MainWindow()
 	KMenu *playlistMenu = new KMenu(i18nc("menu bar", "Play&list"), this);
 	menuBar->addMenu(playlistMenu);
 
-#ifdef HAVE_DVB
+#if HAVE_DVB == 1
 	KMenu *dvbMenu = new KMenu(i18n("&Television"), this);
 	menuBar->addMenu(dvbMenu);
-#endif /* HAVE_DVB */
+#endif /* HAVE_DVB == 1 */
 
 	menu = new KMenu(i18n("&Settings"), this);
 	menuBar->addMenu(menu);
@@ -240,6 +240,7 @@ MainWindow::MainWindow()
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(configureKaffeine()));
 	menu->addAction(collection->addAction("settings_kaffeine", action));
 
+	menuBar->addSeparator();
 	menuBar->addMenu(helpMenu());
 
 	// navigation bar - keep in sync with TabIndex enum!
@@ -252,10 +253,11 @@ MainWindow::MainWindow()
 	tabBar->addTab(KIcon("start-here-kde"), i18n("Start"));
 	tabBar->addTab(KIcon("kaffeine"), i18n("Playback"));
 	tabBar->addTab(KIcon("view-media-playlist"), i18n("Playlist"));
-#ifdef HAVE_DVB
+#if HAVE_DVB == 1
 	tabBar->addTab(KIcon("video-television"), i18n("Television"));
-#endif /* HAVE_DVB */
+#endif /* HAVE_DVB == 1 */
 	tabBar->setShape(KTabBar::RoundedWest);
+	tabBar->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 	connect(tabBar, SIGNAL(currentChanged(int)), this, SLOT(activateTab(int)));
 	navigationBar->addWidget(tabBar);
 
@@ -288,6 +290,7 @@ MainWindow::MainWindow()
 	stackedLayout->addWidget(startTab);
 
 	playerTab = new PlayerTab(mediaWidget);
+	playerTab->activate();
 	tabs.append(playerTab);
 	stackedLayout->addWidget(playerTab);
 
@@ -295,11 +298,13 @@ MainWindow::MainWindow()
 	tabs.append(playlistTab);
 	stackedLayout->addWidget(playlistTab);
 
-#ifdef HAVE_DVB
+#if HAVE_DVB == 1
 	dvbTab = new DvbTab(dvbMenu, collection, mediaWidget);
+	connect(this, SIGNAL(mayCloseApplication(bool*,QWidget*)),
+		dvbTab, SLOT(mayCloseApplication(bool*,QWidget*)));
 	tabs.append(dvbTab);
 	stackedLayout->addWidget(dvbTab);
-#endif /* HAVE_DVB */
+#endif /* HAVE_DVB == 1 */
 
 	currentTabIndex = StartTabId;
 
@@ -334,10 +339,10 @@ MainWindow::MainWindow()
 		QDBusConnection::ExportAllContents);
 	QDBusConnection::sessionBus().registerObject("/TrackList",
 		new MprisTrackListObject(playlistTab, this), QDBusConnection::ExportAllContents);
-#ifdef HAVE_DVB
+#if HAVE_DVB == 1
 	QDBusConnection::sessionBus().registerObject("/Television",
 		new DBusTelevisionObject(dvbTab, this), QDBusConnection::ExportAllContents);
-#endif /* HAVE_DVB */
+#endif /* HAVE_DVB == 1 */
 	QDBusConnection::sessionBus().registerService("org.mpris.kaffeine");
 
 	show();
@@ -397,7 +402,7 @@ void MainWindow::parseArgs()
 		return;
 	}
 
-#ifdef HAVE_DVB
+#if HAVE_DVB == 1
 	if (args->isSet("dumpdvb")) {
 		dvbTab->enableDvbDump();
 	}
@@ -429,7 +434,7 @@ void MainWindow::parseArgs()
 		args->clear();
 		return;
 	}
-#endif /* HAVE_DVB */
+#endif /* HAVE_DVB == 1 */
 
 	if (args->count() > 0) {
 		QList<KUrl> urls;
@@ -598,6 +603,18 @@ void MainWindow::hideCursor()
 	setCursor(Qt::BlankCursor);
 }
 
+void MainWindow::closeEvent(QCloseEvent *event)
+{
+	bool ok = true;
+	emit mayCloseApplication(&ok, this);
+
+	if (!ok) {
+		event->ignore();
+	} else {
+		KMainWindow::closeEvent(event);
+	}
+}
+
 bool MainWindow::event(QEvent *event)
 {
 	bool retVal = KMainWindow::event(event); // this has to be done before calling setVisible()
