@@ -519,13 +519,14 @@ QString MediaWidget::extensionFilter()
 	return QString(
 		// generated from kaffeine.desktop's mime types
 		"*.669 *.aac *.ac3 *.aif *.aifc *.aiff *.anim1 *.anim2 *.anim3 *.anim4 *.anim5 "
-		"*.anim6 *.anim7 *.anim8 *.anim9 *.animj *.asf *.asx *.au *.avi *.divx *.dv "
-		"*.flac *.flc *.fli *.flv *.it *.m15 *.m2t *.m3u *.m3u8 *.m4a *.m4b *.m4v *.med "
-		"*.mka *.mkv *.mng *.mod *.moov *.mov *.mp+ *.mp2 *.mp3 *.mp4 *.mpc *.mpe *.mpeg "
-		"*.mpg *.mpga *.mpp *.mtm *.nsv *.oga *.ogg *.ogm *.ogv *.ogx *.pls *.qt *.qtl "
-		"*.qtvr *.ra *.ram *.rax *.rm *.rmj *.rmm *.rms *.rmvb *.rmx *.rv *.rvx *.s3m "
-		"*.shn *.snd *.spx *.spx *.stm *.tta *.ult *.uni *.vlc *.vob *.voc *.wav *.wax "
-		"*.wma *.wmv *.wmx *.wv *.wvp *.wvx *.xm *.xspf "
+		"*.anim6 *.anim7 *.anim8 *.anim9 *.animj *.asf *.asx *.au *.avi *.bdm *.bdmv "
+		"*.clpi *.cpi *.divx *.dv *.flac *.flc *.fli *.flv *.it *.m15 *.m2t *.m2ts *.m3u "
+		"*.m3u8 *.m4a *.m4b *.m4v *.med *.mka *.mkv *.mng *.mod *.moov *.mov *.mp+ *.mp2 "
+		"*.mp3 *.mp4 *.mpc *.mpe *.mpeg *.mpg *.mpga *.mpl *.mpls *.mpp *.mtm *.mts *.nsv "
+		"*.oga *.ogg *.ogm *.ogv *.ogx *.pls *.qt *.qtl *.qtvr *.ra *.ram *.rax *.rm "
+		"*.rmj *.rmm *.rms *.rmvb *.rmx *.rv *.rvx *.s3m *.shn *.snd *.spx *.stm *.ts "
+		"*.tta *.ult *.uni *.vlc *.vob *.voc *.wav *.wax *.wma *.wmv *.wmx *.wv *.wvp "
+		"*.wvx *.xm *.xspf "
 		// manual entries
 		"*.kaffeine *.iso|") + i18nc("file filter", "Supported Media Files") +
 		"\n*|" + i18nc("file filter", "All Files");
